﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;

// My namespaces
using System.Threading.Tasks;
using System.Security.Cryptography;

namespace WCFServiceWebRoleSmsAuth
{
    public class SmsAuthService : ISmsAuthService
    {
        public async Task<String> CalculateCodeWithIterationNumber(String secretKey, Int64 iterationNumber, Int32 digits = 6)
        {
            return await Task<String>.Run(() =>
            {
                // Impacchettiamo in un array di byte l' iteration number
                Byte[] counter = BitConverter.GetBytes(iterationNumber);

                // Nel caso sia in formato LittleEndian lo capovolgiamo
                // per essere sicuri di usare il valore preso in input
                if (BitConverter.IsLittleEndian)
                    Array.Reverse(counter);

                // Impacchettiamo la chiave segreta in un array di byte
                Byte[] key = Encoding.ASCII.GetBytes(secretKey);

                // Istanziamo l'oggetto HMACSHA1 (Hash-based Message Auth Code with SHA1)
                // specificando come flag di usare la versione gestita dell' algoritmo SHA1
                // e la nostra chiave segreta
                HMACSHA1 hmac = new HMACSHA1(key, true);

                // Computiamo l'hash e lo salviamo in un array di byte
                Byte[] hash = hmac.ComputeHash(counter);

                // Calcoliamo l'ultimo offset
                Int32 offset = hash[hash.Length - 1] & 0xf;

                // Estraiamo il valore intero
                Int32 binary =  ((hash[offset] & 0x7f) << 24)       | 
                                ((hash[offset + 1] & 0xff) << 16)   | 
                                ((hash[offset + 2] & 0xff) << 8)    | 
                                (hash[offset + 3] & 0xff);

                // Calcoliamo il resto della divisione per 10^x 
                // dove x rappresenta il numero di cifre del nostro codice di sicurezza
                // in questo caso saranno 6
                Int32 password = binary % (Int32)Math.Pow(10, digits);

                // facciamo un padding a zero nel caso in cui il risultato
                // precedente abbia meno cifre, e restituiamo il valore
                return password.ToString(new String('0', digits));
            });
        }

        public async Task<String> CalculateCodeWithTime(String secretKey)
        {
            // Istanziamo un oggetto DateTime con i valori dell' UNIX_TIME
            DateTime UNIX_TIME = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);

            // Calcoliamo l'iteration number basandoci sulla differenza tra l' UNIX_TIME
            // e il momento Now espressa in secondi e divisa per 30
            Int64 counter = (Int64)(DateTime.Now - UNIX_TIME).TotalSeconds / 30;

            // Chaimiamo la funzione definita sopra dando come iteration number
            // il valore appena calcolato.
            String returnCode = await CalculateCodeWithIterationNumber(secretKey, counter);

            // Restituiamo il codice di sicurezza
            return returnCode;
        }
    }
}
